c       Truncate hkl file to an ellipse
c
c       The general equation of an ellipse is given below:
c       x**2/a**2 + y**2/b**2 + z**2/c**2 = 1
c       For any ellipse in general a, b, c are the vertices of the ellipse in the x,y,z directions
c       In this case, they correspond to the maximum resolution along h, k, and l directions.
c       X,y,z in the numerator will correspond to distances from origin to reciprocal lattice points
c       along h, k, l (components of d*).  See page 567 of Swokowski Calculus book.
c       Points outside the ellipse will give values greater than 1, and will be discarded.
c       Points inside the llipse will give values less than 1, and will be kept.
c
c
	character orthobox*8,pseudoprecinp*80
	character pdbfile3*80, finfilein*80,finfileout*80
        character junk*80,etruncinp*80,intenstring*58
        character resout*17
        integer refno,reject
c       hi,ki,li, are integer h k l values straight from sca file
        integer hi,ki,li
c       h,k,l are original hkl values floated
        real h,k,l, low,med,hih,alim,blim,clim
c       ho, ko, lo are the orthogonalized h k l values
        real ho,ko,lo
c       hr, kr, lr are the orthogonalized h k l values rotated to put
c       the principle components of diffraction pattern aligned with x,y,z
        real hr,kr,lr
        real pi
        real num, a,b,c,alpha,beta,gamma
        real astar,bstar,cstar
c       alphastar,betastar,gammastar
        real alphastar,betastar,gammastar
c       alphastar betastar gammastar in degrees
        real ad,bd,gd
        real scale, intenso,lambda,sigma
        real hilim, lolim
c       statistics for rejected reflections
        real sumsigreject, avfos, sumfreject,rno
c       variables needed for calculating the volume of the unit cell
        real V,sq1,sq2,sq3,sq4,sqterm
c       elements of orthogonalization matrix
        real l1,l2,l3,m1,m2,m3,n1,n2,n3
c       elements of the principle component matrix output by phaser anisotropy analysis
        real r1c1,r1c2,r1c3,r2c1,r2c2,r2c3,r3c1,r3c2,r3c3
c
        write(*,*) 'Please type in name of input script'
        read(*,'(a80)')etruncinp
        open (2, file=etruncinp,status='old',form='formatted')
        read (2,501)finfilein
        read (2,501)finfileout
 501    format (a80)
        read (2,502)a,b,c,alpha,beta,gamma
 502    format (6f8.2)
        read (2,503)r1c1,r1c2,r1c3,r2c1,r2c2,r2c3,r3c1,r3c2,r3c3
 503    format (9f12.4)
        read (2,503)alim,blim,clim
        resout=finfilein(1:4)//'_reslimit.dat'
        open (7,file=resout,status='new',form='formatted')
c      I am going to sort alim,blim,clim to order low med hih resolution
c   condition 1, a* is the poorest direction
       if((alim.ge.blim).and.(alim.ge.clim))then
          low=alim
          if(blim.gt.clim)then
            med=blim
            hih=clim
          else
            med=clim
            hih=blim
          endif
          goto 60
       endif
c   condition 2, b* is the poorest direction
       if((blim.ge.alim).and.(blim.ge.clim))then
          low=blim
          if(alim.gt.clim)then
            med=alim
            hih=clim
          else
            med=clim
            hih=alim
          endif
          goto 60
       endif
c   condition 3, c* is the poorest direction
       if((clim.ge.alim).and.(clim.ge.blim))then
          low=clim
          if(alim.gt.blim)then
            med=alim
            hih=blim
          else
            med=blim
            hih=alim
          endif
          goto 60
       endif
 60      print*, 'Low, medium and high res limits',low,med,hih 
         write (7,53)hih
 53      format('INCLUDE_RESOLUTION_RANGE=500 ',f8.3)

C   set initial values
        pi=3.141592653589793
        reject=0
        refno=0
        sumfreject=0.0
        sumsigreject=0.0
        avfos=0.0

	open (1,file=finfilein,status='old',form='formatted')
        open (3,file=finfileout,status='new',form='formatted')
        n=0
c       first convert degrees to radians because fortran
c       is so primitive, it cant handle degrees.
          alpha=alpha*pi/180.0
          beta=beta*pi/180.0
          gamma=gamma*pi/180.0
c       calculate volume of unit cell which is needed to 
c       calculate the reciprocal lattice dimensions, astar,bstar,cstar
          sq1=(cos(alpha))**2
          sq2=(cos(beta))**2
          sq3=(cos(gamma))**2
          sq4=2*(cos(alpha))*(cos(beta))*(cos(gamma))
          sqterm=sqrt(1-sq1-sq2-sq3+sq4)
          V=a*b*c*(sqterm)
c       calculate astar and bstar and cstar. see page 33 of stout n jensen
c       or see page 64 of Giacovazzo. Table 2.1
          astar=(b*c*sin(alpha))/V
          bstar=(a*c*sin(beta))/V
          cstar=(a*b*sin(gamma))/V
          print*,'a= ',a,' b= ',b,' c= ',c
          print*,'alpha= ',alpha*180/pi,' beta= ',beta*180/pi,
     1   ' gamma= ',gamma*180/pi
          print*,'astar= ',astar,' bstar= ',bstar,' cstar=',cstar
c       calculate reciprocal cell angles needed for orthogonalization matrix
c       See page 64 of Giacovazzo, table 2.1
          cosalphastar=((cos(beta)*cos(gamma))
     1  -cos(alpha))/sin(beta)*sin(gamma)
          cosbetastar=((cos(alpha)*cos(gamma))
     1  -cos(beta))/sin(alpha)*sin(gamma)
          print*,'cosalpha*=',cosalphastar
         print *, 'reading the fin file now mike'

c      skip over header
 2012   read (1,501,end=999)junk
        if(junk(1:14).eq.'!END_OF_HEADER')go to 2013
        write (3,501)junk
        go to 2012
 2013   write (3,*)'!END_OF_HEADER'
        print*, 'done writing the header'

c  READ IN COORDINATES, END AT LINE 6
   4	read (1,3,err=6)hi,ki,li,intenstring
           read (intenstring(1:11),*)intenso
           read (intenstring(12:22),*)sigma
   3	format(3i6,a58)
         refno=refno+1
         h=float(hi)
         k=float(ki)
         l=float(li)
c        Apply the orthogonalization matrix to the reflections h,k,l
c        to get ho, ko, lo (non-integers but transformed to a cartesian system
c        which is easier to handle when testing whether the reflection is 
c        within an ellipse or not.)
c       Calculate the elements of the orthogonalization matrix 
c       as described on page 68 of Giacovazzo.
c       (l1 l2 l3)  (e1)
c       (m1 m2 m3)  (e2)
c       (n1 n2 n3)  (e3)
          l1=1/a
          l2=0
          l3=0
          m1=-cos(gamma)/(a*sin(gamma))
          m2=1/(b*sin(gamma))
          m3=0
          n1=astar*cosbetastar
          n2=bstar*cosalphastar
          n3=cstar
c       Calculate the h, k, and l components of d* (dstar)
         dhstar=(h*l1)+(k*l2)+(l*l3)
         dkstar=(h*m1)+(k*m2)+(l*m3)
         dlstar=(h*n1)+(k*n2)+(l*n3)
         res=1/sqrt((dhstar**2)+(dkstar**2)+(dlstar**2))
c       write (3,43)dhstar*a,dkstar*b,dlstar*c,intenso,res
  43	format(3f8.2,2f8.1)
c        Now, rotate the reflections so that the strongest diffracting
c        direction is along x, the medium direction is along y, and the
c        weakest direction is along z, according to the phaser anisotropy
c        analysis, which might look like this:
c  Principal components of anisotropic part of B affecting observed amplitudes:
c    eigenB (A^2)     direction cosines (orthogonal coordinates)
c       19.587              0.0000   1.0000   0.0000
c       13.163              1.0000  -0.0000  -0.0000
c      -32.750              0.0000  -0.0000   1.0000
c  Anisotropic deltaB (i.e. range of principal components):  52.337
         hr=(dhstar*r1c1)+(dkstar*r1c2)+(dlstar*r1c3)
         kr=(dhstar*r2c1)+(dkstar*r2c2)+(dlstar*r2c3)
         lr=(dhstar*r3c1)+(dkstar*r3c2)+(dlstar*r3c3)
c       calculate numerators  
          dhstar2=(hr)**2
          dkstar2=(kr)**2
          dlstar2=(lr)**2
c       calculate denominators, here we give our estimates of 
c       max resolutions along weak, medium, and strong directions
c      MUST BE IN THIS ORDER IF YOU ARE USING THE direction cosines
c      FROM the PHASER ANISOTROPY OUTPUT
c       these resolution limits are the  vertices of the ellipse
          vrtxh2=1/low**2
          vrtxk2=1/med**2
          vrtxl2=1/hih**2
c       Now I want to test whether or not ho,ko,lo
c       are points on the ellipse .  
          dist=(dhstar2/vrtxh2)+(dkstar2/vrtxk2)+(dlstar2/vrtxl2)
          res=sqrt(dhstar2+dkstar2+dlstar2)
          if(dist.gt.1)then
           reject=reject+1
           sumfreject=sumfreject+intenso
           sumsigreject=sumsigreject+sigma
           avfos=sumfreject/sumsigreject
           goto 4
          endif
c       Print the reflections that lie within the ellipse
        write (3,3)hi,ki,li,intenstring
          go to 4
   6   print*, refno,' reflections were input'
 2014   write (3,*)'!END_OF_DATA'

        write(*,*) 'Please type in name of pseudoprec script'
        read(*,'(a80)')pseudoprecinp
        open (4, file=pseudoprecinp,status='new',form='formatted')
        write (4,34)refno
  30    format(f12.2)
  34    format(i9)
       print*, reject,' were rejected for being outside the ellipsoid'
        write (4,34)reject
       print*, 'These had an average F/sigma = ', avfos
           rno=refno-reject
        write (4,31)avfos
  31    format(f12.2)
       print*, rno, 'reflections remain after ellipsoidal truncation'
        write (4,30)rno

 999	stop
	end
